<?php
/*
 * This file is part of EC-CUBE
 *
 * Copyright(c) 2000-2010 LOCKON CO.,LTD. All Rights Reserved.
 *
 * http://www.lockon.co.jp/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

// {{{ requires
require_once(CLASS_PATH . "pages/shopping/LC_Page_Shopping_Complete.php");
require_once(MODULE_PATH . "mdl_dchk/class/helper/SC_Helper_Mail_Ex_Dchk.php");

/**
 * ご注文完了 のページクラス(拡張).
 *
 * LC_Page_Shopping_Complete をカスタマイズする場合はこのクラスを編集する.
 *
 * @package Page
 * @author LOCKON CO.,LTD.
 * @version $Id: LC_Page_Shopping_Complete_Ex.php 18734 2010-06-22 08:45:33Z nanasess $
 */
class LC_Page_Shopping_Complete_Ex extends LC_Page_Shopping_Complete {

    // }}}
    // {{{ functions

    /**
     * Page を初期化する.
     *
     * @return void
     */
    function init() {
        parent::init();
    }

    /**
     * Page のプロセス.
     *
     * @return void
     */
    function process() {
        global $objCampaignSess;

        $conn = new SC_DBConn();
        $objView = new SC_SiteView();
        $this->objSiteSess = new SC_SiteSession();
        $this->objCartSess = new SC_CartSession();
        $this->objCampaignSess = new SC_CampaignSession();
        $objSiteInfo = $objView->objSiteInfo;
        $this->arrInfo = $objSiteInfo->data;
        $this->objCustomer = new SC_Customer();
        $mailHelper = new SC_Helper_Mail_Ex_Dchk();

        // 前のページで正しく登録手続きが行われたか判定
        SC_Utils_Ex::sfIsPrePage($this->objSiteSess);
        // ユーザユニークIDの取得と購入状態の正当性をチェック
        $uniqid = SC_Utils_Ex::sfCheckNormalAccess($this->objSiteSess, $this->objCartSess);
        if ($uniqid != "") {

            // 完了処理
            $objQuery = new SC_Query();
            $objQuery->begin();
            if($_SESSION["payment_id"] > 0){
                $order_id = $this->lfDoCompleteMdl($objQuery, $uniqid);
            }else{
                $order_id = $this->lfDoComplete($objQuery, $uniqid);
            }
            $objQuery->commit();

            // セッションに保管されている情報を更新する
            $this->objCustomer->updateSession();

            // 完了メール送信
            if($order_id != "") {
                $mailHelper->sfSendOrderMail($order_id, '1');
            }

            // その他情報の取得
            $arrResults = $objQuery->getAll("SELECT memo02, memo05 FROM dtb_order WHERE order_id = ? ", array($order_id));

            if (count($arrResults) > 0) {
                if (isset($arrResults[0]["memo02"]) || isset($arrResults[0]["memo05"])) {
                    // 完了画面で表示する決済内容
                    $arrOther = unserialize($arrResults[0]["memo02"]);
                    // 完了画面から送信する決済内容
                    $arrModuleParam = unserialize($arrResults[0]["memo05"]);

                    // データを編集
                    foreach($arrOther as $key => $val){
                        // URLの場合にはリンクつきで表示させる
                        if (preg_match('/^(https?|ftp)(:\/\/[-_.!~*\'()a-zA-Z0-9;\/?:\@&=+\$,%#]+)$/', $val["value"])) {
                            $arrOther[$key]["value"] = "<a href='#' onClick=\"window.open('". $val["value"] . "'); \" >" . $val["value"] ."</a>";
                        }
                    }

                    $this->arrOther = $arrOther;
                    $this->arrModuleParam = $arrModuleParam;
                }
            }
            
            // アフィリエイト用コンバージョンタグの設定
            $this->tpl_conv_page = AFF_SHOPPING_COMPLETE;
            $this->tpl_aff_option = "order_id=$order_id";
            //合計価格の取得
            $total = $objQuery->get("dtb_order", "total", "order_id = ? ", array($order_id));
            if($total != "") {
                $this->tpl_aff_option.= "|total=$total";
            }

            // TradeSafe連携用
            if (function_exists('sfTSRequest')) {
                sfTSRequest($order_id);
            }
        }

        // キャンペーンからの遷移かチェック
        $this->is_campaign = $this->objCampaignSess->getIsCampaign();
        $this->campaign_dir = $this->objCampaignSess->getCampaignDir();

        $objView->assignobj($this);
        // フレームを選択(キャンペーンページから遷移なら変更)
        $this->objCampaignSess->pageView($objView);

        // セッション開放
        $this->objCampaignSess->delCampaign();
    }

    /**
     * デストラクタ.
     *
     * @return void
     */
    function destroy() {
        parent::destroy();
    }

    /**
     * Page のプロセス(モバイル).
     *
     * @return void
     */
    function mobileProcess() {
        $conn = new SC_DBConn();
        $objView = new SC_MobileView();
        $this->objSiteSess = new SC_SiteSession();
        $this->objCartSess = new SC_CartSession();
        $objSiteInfo = $objView->objSiteInfo;
        $this->arrInfo = $objSiteInfo->data;
        $this->objCustomer = new SC_Customer();
        $mailHelper = new SC_Helper_Mail_Ex_Dchk();

        // 前のページで正しく登録手続きが行われたか判定
        SC_Utils_Ex::sfIsPrePage($this->objSiteSess, true);
        // ユーザユニークIDの取得と購入状態の正当性をチェック
        $uniqid = SC_Utils_Ex::sfCheckNormalAccess($this->objSiteSess, $this->objCartSess);
        if ($uniqid != "") {

            // 完了処理
            $objQuery = new SC_Query();
            $objQuery->begin();
            if($_SESSION["payment_id"] > 0){
                $order_id = $this->lfDoCompleteMdl($objQuery, $uniqid);
            }else{
                $order_id = $this->lfDoComplete($objQuery, $uniqid);
            }
            $objQuery->commit();

            // セッションに保管されている情報を更新する
            $this->objCustomer->updateSession();

            // 完了メール送信
            if($order_id != "") {
                $mailHelper->sfSendOrderMail($order_id, '2');
            }

            //その他情報の取得
            $other_data = $objQuery->get("dtb_order", "memo02", "order_id = ? ", array($order_id));
            if($other_data != "") {
                $arrOther = unserialize($other_data);

                // データを編集
                foreach($arrOther as $key => $val){
                    // URLの場合にはリンクつきで表示させる
                    if (preg_match('/^(https?|ftp)(:\/\/[-_.!~*\'()a-zA-Z0-9;\/?:\@&=+\$,%#]+)$/', $val["value"])) {
                        $arrOther[$key]["value"] = "<a href='". $val["value"]. "'>". $val["value"]. "</a>";
                    }
                }

                $this->arrOther = $arrOther;

            }

            // アフィリエイト用コンバージョンタグの設定
            $this->tpl_conv_page = AFF_SHOPPING_COMPLETE;
            $this->tpl_aff_option = "order_id=$order_id";
            //合計価格の取得
            $total = $objQuery->get("dtb_order", "total", "order_id = ? ", array($order_id));
            if($total != "") {
                $this->tpl_aff_option.= "|total=$total";
            }

            // TS連携モジュールの実行
            if (function_exists('sfTSRequest')) {
                sfTSRequest($order_id);
            }
        }

        $objView->assignobj($this);
        $objView->display(SITE_FRAME);
    }

    
    /**
     * 購入完了処理
     *
     * @param object $objQuery
     * @param string $uniqid
     * @return string $order_id
     */
    function lfDoCompleteMdl(&$objQuery, $uniqid) {
        $objDb = new SC_Helper_DB_Ex();
        
        // 受注テーブルの読込
        $order_id = $_SESSION["order_id"];
        $arrData = $objQuery->SELECT("*","dtb_order","order_id = ?",array($order_id));
        $arrData = $arrData[0];

        // 会員情報登録処理
        if ($this->objCustomer->isLoginSuccess(true)) {
            // 購入集計を顧客テーブルに反映
            $this->lfSetCustomerPurchase($this->objCustomer->getValue('customer_id'), $arrData, $objQuery);
        }

        // 受注一時テーブルの情報を削除する。
        $this->lfDeleteTempOrder($objQuery, $uniqid);
        // キャンペーンからの遷移の場合登録する。
        if (!defined("MOBILE_SITE")) {
            if($this->objCampaignSess->getIsCampaign() and $this->objCartSess->chkCampaign($this->objCampaignSess->getCampaignId())) {
                $this->lfRegistCampaignOrder($objQuery, $objCampaignSess, $order_id);
            }
        }

        // セッションカート内の商品を削除する。
        $this->objCartSess->delAllProducts();
        // 注文一時IDを解除する。
        $this->objSiteSess->unsetUniqId();
        unset($_SESSION["order_id"],$_SESSION["payment_id"]);
        
        return $order_id;
    }

    // 受注テーブルへ登録
    function lfUpdateOrder($objQuery, $arrData, $objCampaignSess = null) {
        $sqlval = array('status' => ORDER_NEW , 'update_date' => 'now()');
        
        // キャンペーンID
        if (!defined("MOBILE_SITE")) {
            if($objCampaignSess->getIsCampaign()) $sqlval['campaign_id'] = $objCampaignSess->getCampaignId();
        }

        // UPDATEの実行
        $objQuery->update("dtb_order", $sqlval,"order_id = ? ", array($_SESSION["order_id"]));

        return $_SESSION["order_id"];
    }


}
?>
