<?php
/*
 * This file is part of EC-CUBE
 *
 * Copyright(c) 2000-2010 LOCKON CO.,LTD. All Rights Reserved.
 *
 * http://www.lockon.co.jp/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

// {{{ requires
require_once(CLASS_PATH . "pages/shopping/LC_Page_Shopping_Confirm.php");

/**
 * 入力内容確認 のページクラス(拡張).
 *
 * LC_Page_Shopping_Confirm をカスタマイズする場合はこのクラスを編集する.
 *
 * @package Page
 */
class LC_Page_Shopping_Confirm_Ex extends LC_Page_Shopping_Confirm {

    // }}}
    // {{{ functions

    /**
     * Page を初期化する.
     *
     * @return void
     */
    function init() {
        parent::init();
    }


    /**
     * Page のプロセス.
     *
     * @return void
     */
    function process() {
        global $objCampaignSess;

        $objView = new SC_SiteView();
        $this->objCartSess = new SC_CartSession();
        $objSiteInfo = $objView->objSiteInfo;
        $objSiteSess = new SC_SiteSession();
        $objCampaignSess = new SC_CampaignSession();
        $this->objCustomer = new SC_Customer();
        $arrInfo = $objSiteInfo->data;
        $objQuery = new SC_Query();
        $objDb = new SC_Helper_DB_Ex();
        
        
        // 前のページで正しく登録手続きが行われた記録があるか判定
        SC_Utils_Ex::sfIsPrePage($objSiteSess);

        // ユーザユニークIDの取得と購入状態の正当性をチェック
        $uniqid = SC_Utils_Ex::sfCheckNormalAccess($objSiteSess, $this->objCartSess);
        $this->tpl_uniqid = $uniqid;

        // カート集計処理
        $objDb->sfTotalCart($this, $this->objCartSess, $arrInfo);
        if (strlen($this->tpl_message) >= 1) {
            SC_Utils_Ex::sfDispSiteError(SOLD_OUT, '', true);
        }
        // 一時受注テーブルの読込
        $arrData = $objDb->sfGetOrderTemp($uniqid);
        // カート集計を元に最終計算
        $arrData = $objDb->sfTotalConfirm($arrData, $this, $this->objCartSess, $arrInfo, $this->objCustomer, $objCampaignSess);
        // キャンペーンからの遷移で送料が無料だった場合の処理
        if($objCampaignSess->getIsCampaign()) {
            $deliv_free_flg = $objQuery->get("dtb_campaign", "deliv_free_flg", "campaign_id = ?", array($objCampaignSess->getCampaignId()));
            // 送料無料が設定されていた場合
            if($deliv_free_flg) {
                $arrData['payment_total'] -= $arrData['deliv_fee'];
                $arrData['deliv_fee'] = 0;
            }
        }


        // カート内の商品の売り切れチェック
        $this->objCartSess->chkSoldOut($this->objCartSess->getCartList());

        // 会員ログインチェック
        if($this->objCustomer->isLoginSuccess()) {
            $this->tpl_login = '1';
            $this->tpl_user_point = $this->objCustomer->getValue('point');
        }

        // 決済区分を取得する
        $payment_type = "";
        if($objDb->sfColumnExists("dtb_payment", "memo01")){
            // MEMO03に値が入っている場合には、モジュール追加されたものとみなす
            $sql = "SELECT memo03 FROM dtb_payment WHERE payment_id = ?";
            $arrPayment = $objQuery->getAll($sql, array($arrData['payment_id']));
            $payment_type = $arrPayment[0]["memo03"];
        }
        $this->payment_type = $payment_type;


        if (!isset($_POST['mode'])) $_POST['mode'] = "";

        switch($_POST['mode']) {
        // 前のページに戻る
        case 'return':
            // 正常な推移であることを記録しておく
            $objSiteSess->setRegistFlag();
            $this->sendRedirect($this->getLocation(URL_SHOP_PAYMENT));
            exit;
            break;
        case 'confirm':
            // この時点で注文番号を確保しておく（クレジット、コンビニ決済で必要なため）
            // postgresqlとmysqlとで処理を分ける
            if (DB_TYPE == "pgsql") {
                $order_id = $objQuery->nextval("dtb_order","order_id");
            }elseif (DB_TYPE == "mysql") {
                $order_id = $objQuery->get_auto_increment("dtb_order");
            }
            $arrData["order_id"] = $order_id;
            
            // セッション情報を保持
            $arrData['session'] = serialize($_SESSION);

            // 集計結果を受注一時テーブルに反映
            $objDb->sfRegistTempOrder($uniqid, $arrData);
            // 正常に登録されたことを記録しておく
            $objSiteSess->setRegistFlag();

            // 決済方法により画面切替
            if($payment_type != "") {
                //一旦受注情報を保存
                $this->lfDoComplete($objQuery, $uniqid );
                
                // TODO 決済方法のモジュールは Plugin として実装したい
                SC_Session::SetSession("payment_id",$arrData['payment_id']);
                $_SESSION["payment_id"] = $arrData['payment_id'];
                $_SESSION["order_id"] = $order_id;
                $this->sendRedirect($this->getLocation(URL_SHOP_MODULE));
            }else{
                SC_Session::SetSession("payment_id",null);
                $this->sendRedirect($this->getLocation(URL_SHOP_COMPLETE));
            }
            exit;
            break;
        default:
            break;
        }

        $this->arrData = $arrData;
        $this->arrInfo = $arrInfo;
        $objView->assignobj($this);
        // フレームを選択(キャンペーンページから遷移なら変更)
        $objCampaignSess->pageView($objView);
    }

    /**
     * モバイルページを初期化する.
     *
     * @return void
     */
    function mobileInit() {
        $this->init();
    }

    /**
     * Page のプロセス(モバイル).
     *
     * @return void
     */
    function mobileProcess() {
        $objView = new SC_MobileView();
        $this->objCartSess = new SC_CartSession();
        $objSiteInfo = $objView->objSiteInfo;
        $objSiteSess = new SC_SiteSession();
        $this->objCustomer = new SC_Customer();
        $arrInfo = $objSiteInfo->data;
        $objQuery = new SC_Query();
        $objDb = new SC_Helper_DB_Ex();
        $this->objCartSess = $this->objCartSess;
        

        // 前のページで正しく登録手続きが行われた記録があるか判定
        SC_Utils_Ex::sfIsPrePage($objSiteSess, true);

        // ユーザユニークIDの取得と購入状態の正当性をチェック
        $uniqid = SC_Utils_Ex::sfCheckNormalAccess($objSiteSess, $this->objCartSess);
        $this->tpl_uniqid = $uniqid;

        // カート集計処理
        $objDb->sfTotalCart($this, $this->objCartSess, $arrInfo);
        // 一時受注テーブルの読込
        $arrData = $objDb->sfGetOrderTemp($uniqid);
        // カート集計を元に最終計算
        $arrData = $objDb->sfTotalConfirm($arrData, $this, $this->objCartSess, $arrInfo, $this->objCustomer);

        // カート内の商品の売り切れチェック
        $this->objCartSess->chkSoldOut($this->objCartSess->getCartList());

        // 会員ログインチェック
        if($this->objCustomer->isLoginSuccess(true)) {
            $this->tpl_login = '1';
            $this->tpl_user_point = $this->objCustomer->getValue('point');
        }

        // 決済区分を取得する
        $payment_type = "";
        if($objDb->sfColumnExists("dtb_payment", "memo01")){
            // MEMO03に値が入っている場合には、モジュール追加されたものとみなす
            $sql = "SELECT memo03 FROM dtb_payment WHERE payment_id = ?";
            $arrPayment = $objQuery->getAll($sql, array($arrData['payment_id']));
            $payment_type = $arrPayment[0]["memo03"];
        }
        $this->payment_type = $payment_type;

        if (!isset($_POST['mode'])) $_POST['mode'] = "";

        switch($_POST['mode']) {
            // 前のページに戻る
        case 'return':
            // 正常な推移であることを記録しておく
            $objSiteSess->setRegistFlag();
            $this->sendRedirect($this->getLocation(MOBILE_URL_SHOP_PAYMENT), true);
            exit;
            break;
        case 'confirm':
            // この時点で注文番号を確保しておく（クレジット、コンビニ決済で必要なため）
            // postgresqlとmysqlとで処理を分ける
            if (DB_TYPE == "pgsql") {
                $order_id = $objQuery->nextval("dtb_order","order_id");
            }elseif (DB_TYPE == "mysql") {
                $order_id = $objQuery->get_auto_increment("dtb_order");
            }
            $arrData["order_id"] = $order_id;

            // セッション情報を保持
            $arrData['session'] = serialize($_SESSION);

            // 集計結果を受注一時テーブルに反映
            $objDb->sfRegistTempOrder($uniqid, $arrData);
            // 正常に登録されたことを記録しておく
            $objSiteSess->setRegistFlag();

            // 決済方法により画面切替
            if($payment_type != "") {
                $this->lfDoComplete($objQuery, $uniqid);
                SC_Session::SetSession("payment_id",$arrData['payment_id']);
                $_SESSION["payment_id"] = $arrData['payment_id'];
                $_SESSION["order_id"]   = $order_id;
                $this->sendRedirect($this->getLocation(MOBILE_URL_SHOP_MODULE), true);
            }else{
                SC_Session::SetSession("payment_id",null);
                $this->sendRedirect($this->getLocation(MOBILE_URL_SHOP_COMPLETE), true);
            }
            exit;
            break;
        default:
            break;
        }
        $this->arrData = $arrData;
        $this->arrInfo = $arrInfo;
        $objView->assignobj($this);
        $objView->display(SITE_FRAME);
    }
    
    /**
     * 購入完了処理
     *
     * @param object $objQuery
     * @param string $uniqid
     * @return string $order_id
     */
    function lfDoComplete(&$objQuery, $uniqid) {
        $objDb = new SC_Helper_DB_Ex();

        // 一時受注テーブルの読込
        $arrData = $objDb->sfGetOrderTemp($uniqid);

        // 会員情報登録処理
        if ($this->objCustomer->isLoginSuccess(true)) {
            // 新お届け先の登録
            $this->lfSetNewAddr($uniqid, $this->objCustomer->getValue('customer_id'));
        } else {
            //購入時強制会員登録
            switch(PURCHASE_CUSTOMER_REGIST) {
            //無効
            case '0':
                // 購入時会員登録
                if(isset($arrData['member_check']) && $arrData['member_check'] == '1') {
                    // 会員登録
                    $customer_id = $this->lfRegistCustomer($arrData, $this->arrInfo);
                }
                break;
            //有効
            case '1':
                // 会員登録
                $customer_id = $this->lfRegistCustomer($arrData, $this->arrInfo);
                break;
            }

        }
        // 一時テーブルを受注テーブルに格納する
        if (defined("MOBILE_SITE")) {
            $order_id = $this->lfRegistOrder($objQuery, $arrData);
        } else {
            $order_id = $this->lfRegistOrder($objQuery, $arrData, $objCampaignSess);
        }
        // カート商品を受注詳細テーブルに格納する
        $this->lfRegistOrderDetail($objQuery, $order_id, $this->objCartSess);
        // キャンペーンからの遷移の場合登録する。
        if (!defined("MOBILE_SITE") and is_object($objCampaignSess)) {
            if($objCampaignSess->getIsCampaign() and $this->objCartSess->chkCampaign($objCampaignSess->getCampaignId())) {
                $this->lfRegistCampaignOrder($objQuery, $objCampaignSess, $order_id);
            }
        }
        return $order_id;
    }

    // 受注テーブルへ登録
    function lfRegistOrder($objQuery, $arrData, $objCampaignSess = null) {
        $sqlval = $arrData;

        // 受注テーブルに書き込まない列を除去
        unset($sqlval['mailmaga_flg']);		// メルマガチェック
        unset($sqlval['deliv_check']);		// 別のお届け先チェック
        unset($sqlval['point_check']);		// ポイント利用チェック
        unset($sqlval['member_check']);		// 購入時会員チェック
        unset($sqlval['password']);			// ログインパスワード
        unset($sqlval['reminder']);			// リマインダー質問
        unset($sqlval['reminder_answer']);	// リマインダー答え
        unset($sqlval['mail_flag']);		// メールフラグ
        unset($sqlval['session']);		    // セッション情報

        // 注文ステータス:指定が無ければ新規受付に設定
        if($sqlval["status"] == ""){
            $sqlval['status'] = '1';
        }

        // 別のお届け先を指定していない場合、お届け先に登録住所をコピーする。
        if($arrData["deliv_check"] == "-1") {
            $sqlval['deliv_name01'] = $arrData['order_name01'];
            $sqlval['deliv_name02'] = $arrData['order_name02'];
            $sqlval['deliv_kana01'] = $arrData['order_kana01'];
            $sqlval['deliv_kana02'] = $arrData['order_kana02'];
            $sqlval['deliv_pref'] = $arrData['order_pref'];
            $sqlval['deliv_zip01'] = $arrData['order_zip01'];
            $sqlval['deliv_zip02'] = $arrData['order_zip02'];
            $sqlval['deliv_addr01'] = $arrData['order_addr01'];
            $sqlval['deliv_addr02'] = $arrData['order_addr02'];
            $sqlval['deliv_tel01'] = $arrData['order_tel01'];
            $sqlval['deliv_tel02'] = $arrData['order_tel02'];
            $sqlval['deliv_tel03'] = $arrData['order_tel03'];
        }

        $order_id = $arrData['order_id'];		// 注文番号
        $sqlval['create_date'] = 'now()';		// 受注日
        $sqlval['status'] = ORDER_PENDING;

        // キャンペーンID
        if (!defined("MOBILE_SITE") and is_object($objCampaignSess)) {
            if($objCampaignSess->getIsCampaign()) $sqlval['campaign_id'] = $objCampaignSess->getCampaignId();
        }

        // ゲットの値をインサート
        //$sqlval = lfGetInsParam($sqlval);

        // INSERTの実行
        $objQuery->insert("dtb_order", $sqlval);

        return $order_id;
    }

    // 受注詳細テーブルへ登録
    function lfRegistOrderDetail(&$objQuery, $order_id) {
        $objDb = new SC_Helper_DB_Ex();
        // カート内情報の取得
        $arrCart = $this->objCartSess->getCartList();
        $max = count($arrCart);

        // 既に存在する詳細レコードを消しておく。
        $objQuery->delete("dtb_order_detail", "order_id = $order_id");

        // 規格名一覧
        $arrClassName = $objDb->sfGetIDValueList("dtb_class", "class_id", "name");
        // 規格分類名一覧
        $arrClassCatName = $objDb->sfGetIDValueList("dtb_classcategory", "classcategory_id", "name");

        for ($i = 0; $i < $max; $i++) {
            // 商品規格情報の取得
            $arrData = $objDb->sfGetProductsClass($arrCart[$i]['id']);

            // 存在する商品のみ表示する。
            if($arrData != "") {
                $sqlval['order_id'] = $order_id;
                $sqlval['product_id'] = $arrCart[$i]['id'][0];
                $sqlval['classcategory_id1'] = $arrCart[$i]['id'][1];
                $sqlval['classcategory_id2'] = $arrCart[$i]['id'][2];
                $sqlval['product_name'] = $arrData['name'];
                $sqlval['product_code'] = $arrData['product_code'];
                $sqlval['classcategory_name1'] = $arrClassCatName[$arrData['classcategory_id1']];
                $sqlval['classcategory_name2'] = $arrClassCatName[$arrData['classcategory_id2']];
                $sqlval['point_rate'] = $arrCart[$i]['point_rate'];
                $sqlval['price'] = $arrCart[$i]['price'];
                $sqlval['quantity'] = $arrCart[$i]['quantity'];
                $this->lfReduceStock($objQuery, $arrCart[$i]['id'], $arrCart[$i]['quantity']);
                // INSERTの実行
                $objQuery->insert("dtb_order_detail", $sqlval);
            } else {
                if (defined("MOBILE_SITE")) {
                    SC_Utils_Ex::sfDispSiteError(CART_NOT_FOUND, "", false, "", true);
                } else {
                    SC_Utils_Ex::sfDispSiteError(CART_NOT_FOUND);
                }
            }
        }
    }
    // 受注一時テーブルの住所が登録済みテーブルと異なる場合は、別のお届け先に追加する
    function lfSetNewAddr($uniqid, $customer_id) {
        $objQuery = new SC_Query();
        $diff = false;
        $find_same = false;

        $col = "deliv_name01,deliv_name02,deliv_kana01,deliv_kana02,deliv_tel01,deliv_tel02,deliv_tel03,deliv_zip01,deliv_zip02,deliv_pref,deliv_addr01,deliv_addr02";
        $where = "order_temp_id = ?";
        $arrRet = $objQuery->select($col, "dtb_order_temp", $where, array($uniqid));

        // 要素名のdeliv_を削除する。
        foreach($arrRet[0] as $key => $val) {
            $keyname = ereg_replace("^deliv_", "", $key);
            $arrNew[$keyname] = $val;
        }

        // 会員情報テーブルとの比較
        $col = "name01,name02,kana01,kana02,tel01,tel02,tel03,zip01,zip02,pref,addr01,addr02";
        $where = "customer_id = ?";
        $arrCustomerAddr = $objQuery->select($col, "dtb_customer", $where, array($customer_id));

        // 会員情報の住所と異なる場合
        if($arrNew != $arrCustomerAddr[0]) {
            // 別のお届け先テーブルの住所と比較する
            $col = "name01,name02,kana01,kana02,tel01,tel02,tel03,zip01,zip02,pref,addr01,addr02";
            $where = "customer_id = ?";
            $arrOtherAddr = $objQuery->select($col, "dtb_other_deliv", $where, array($customer_id));

            foreach($arrOtherAddr as $arrval) {
                if($arrNew == $arrval) {
                    // すでに同じ住所が登録されている
                    $find_same = true;
                }
            }

            if(!$find_same) {
                $diff = true;
            }
        }

        // 新しいお届け先が登録済みのものと異なる場合は別のお届け先テーブルに登録する
        if($diff) {
            $sqlval = $arrNew;
            $sqlval['customer_id'] = $customer_id;
            $objQuery->insert("dtb_other_deliv", $sqlval);
        }
    }

    // 在庫を減らす処理
    function lfReduceStock(&$objQuery, $arrID, $quantity) {
        $where = "product_id = ? AND classcategory_id1 = ? AND classcategory_id2 = ?";
        $arrRet = $objQuery->select("stock, stock_unlimited", "dtb_products_class", $where, $arrID);

        // 売り切れエラー
        if(($arrRet[0]['stock_unlimited'] != '1' && $arrRet[0]['stock'] < $quantity) || $quantity == 0) {
            $objQuery->rollback();
            SC_Utils_Ex::sfDispSiteError(SOLD_OUT, "", true);
        // 無制限の場合、在庫はNULL
        } elseif($arrRet[0]['stock_unlimited'] == '1') {
            $sqlval['stock'] = null;
            $objQuery->update("dtb_products_class", $sqlval, $where, $arrID);
        // 在庫を減らす
        } else {
            $sqlval['stock'] = ($arrRet[0]['stock'] - $quantity);
            if($sqlval['stock'] == "") {
                $sqlval['stock'] = '0';
            }
            $objQuery->update("dtb_products_class", $sqlval, $where, $arrID);
        }
    }

}